# Deadrun

#
# Import the ParFlow TCL package
#
#  Fully saturated steady state run using the IMPES solver.  In this folder, I included the mask pfb file from another (unsaturated) run that uses the same solid file so that the land surface could be extracted, and the pressure at the land surface for this run visualized.  

lappend auto_path $env(PARFLOW_DIR)/bin 
package require parflow
namespace import Parflow::*


#-----------------------------------------------------------------------------
# File input version number
#-----------------------------------------------------------------------------
pfset FileVersion 4

#-----------------------------------------------------------------------------
# Process Topology
#-----------------------------------------------------------------------------

pfset Process.Topology.P        1
pfset Process.Topology.Q        1
pfset Process.Topology.R        1

#-----------------------------------------------------------------------------
# Computational Grid
#-----------------------------------------------------------------------------
pfset ComputationalGrid.Lower.X                0.0
pfset ComputationalGrid.Lower.Y                0.0
pfset ComputationalGrid.Lower.Z                0.0

pfset ComputationalGrid.DX	               100.
pfset ComputationalGrid.DY                     100.
pfset ComputationalGrid.DZ	                 1.

pfset ComputationalGrid.NX                      90
pfset ComputationalGrid.NY                      80
pfset ComputationalGrid.NZ                      339

#---------------------------------------------------------
# The Names of the GeomInputs
#---------------------------------------------------------

pfset GeomInput.Names                 "solidinput"
pfset GeomInput.solidinput.InputType  SolidFile
pfset GeomInput.solidinput.GeomNames  domain
pfset GeomInput.solidinput.FileName   deadrun.pfsol


pfset Geom.domain.Patches             "z-upper z-lower x-lower x-upper \
                                      y-lower y-upper"

#-----------------------------------------------------------------------------
# Perm
#-----------------------------------------------------------------------------
pfset Geom.Perm.Names "domain"

pfset Geom.domain.Perm.Type Constant
pfset Geom.domain.Perm.Value 100.

pfset Perm.TensorType               TensorByGeom

pfset Geom.Perm.TensorByGeom.Names  "domain"

pfset Geom.domain.Perm.TensorValX  1.0
pfset Geom.domain.Perm.TensorValY  1.0
pfset Geom.domain.Perm.TensorValZ  1.0

#-----------------------------------------------------------------------------
# Specific Storage
#-----------------------------------------------------------------------------
# specific storage does not figure into the impes (fully sat) case but we still
# need a key for it

pfset SpecificStorage.Type            Constant
pfset SpecificStorage.GeomNames       ""
pfset Geom.domain.SpecificStorage.Value 1.0e-4

#-----------------------------------------------------------------------------
# Phases
#-----------------------------------------------------------------------------

pfset Phase.Names "water"

pfset Phase.water.Density.Type	Constant
pfset Phase.water.Density.Value	1.0

pfset Phase.water.Viscosity.Type	Constant
pfset Phase.water.Viscosity.Value	1.0

#-----------------------------------------------------------------------------
# Contaminants
#-----------------------------------------------------------------------------
pfset Contaminants.Names			""


#-----------------------------------------------------------------------------
# Gravity
#-----------------------------------------------------------------------------

pfset Gravity				1.0

#-----------------------------------------------------------------------------
# Setup timing info
#-----------------------------------------------------------------------------

pfset TimingInfo.BaseUnit		1.0
pfset TimingInfo.StartCount		-1
pfset TimingInfo.StartTime		0.0
pfset TimingInfo.StopTime            0.0
pfset TimingInfo.DumpInterval	       -1

#-----------------------------------------------------------------------------
# Porosity
#-----------------------------------------------------------------------------

pfset Geom.Porosity.GeomNames          domain

pfset Geom.domain.Porosity.Type    Constant
pfset Geom.domain.Porosity.Value   0.390

#-----------------------------------------------------------------------------
# Domain
#-----------------------------------------------------------------------------
pfset Domain.GeomName domain

#-----------------------------------------------------------------------------
# Mobility
#-----------------------------------------------------------------------------
pfset Phase.water.Mobility.Type        Constant
pfset Phase.water.Mobility.Value       1.0


#-----------------------------------------------------------------------------
# Wells
#-----------------------------------------------------------------------------
pfset Wells.Names ""


#-----------------------------------------------------------------------------
# Time Cycles
#-----------------------------------------------------------------------------
pfset Cycle.Names constant
pfset Cycle.constant.Names		"alltime"
pfset Cycle.constant.alltime.Length	 1
pfset Cycle.constant.Repeat		-1

#-----------------------------------------------------------------------------
# Boundary Conditions: Pressure
#-----------------------------------------------------------------------------
pfset BCPressure.PatchNames          "z-upper z-lower x-lower x-upper \
                                      y-lower y-upper"

pfset Patch.y-lower.BCPressure.Type			DirEquilRefPatch
pfset Patch.y-lower.BCPressure.Cycle			"constant"
pfset Patch.y-lower.BCPressure.RefGeom			domain
pfset Patch.y-lower.BCPressure.RefPatch			z-lower
pfset Patch.y-lower.BCPressure.alltime.Value		300.0  

pfset Patch.y-upper.BCPressure.Type			DirEquilRefPatch
pfset Patch.y-upper.BCPressure.Cycle			"constant"
pfset Patch.y-upper.BCPressure.RefGeom			domain
pfset Patch.y-upper.BCPressure.RefPatch			z-lower
pfset Patch.y-upper.BCPressure.alltime.Value		200.0  #200 m is added to the elevation to account for the aquifer thickness

pfset Patch.x-lower.BCPressure.Type			FluxConst
pfset Patch.x-lower.BCPressure.Cycle			"constant"
pfset Patch.x-lower.BCPressure.alltime.Value		0.0

pfset Patch.x-upper.BCPressure.Type			FluxConst
pfset Patch.x-upper.BCPressure.Cycle			"constant"
pfset Patch.x-upper.BCPressure.alltime.Value		0.0

pfset Patch.z-lower.BCPressure.Type			FluxConst
pfset Patch.z-lower.BCPressure.Cycle			"constant"
pfset Patch.z-lower.BCPressure.alltime.Value		0.0

pfset Patch.z-upper.BCPressure.Type			        FluxConst
pfset Patch.z-upper.BCPressure.Cycle			"constant"
pfset Patch.z-upper.BCPressure.alltime.Value		0.0

#---------------------------------------------------------
# Topo slopes in x-direction
#---------------------------------------------------------
# topo slopes do not figure into the impes (fully sat) case but we still
# need keys for them

pfset TopoSlopesX.Type "Constant"
pfset TopoSlopesX.GeomNames ""

pfset TopoSlopesX.Geom.domain.Value 0.0

#---------------------------------------------------------
# Topo slopes in y-direction
#---------------------------------------------------------

pfset TopoSlopesY.Type "Constant"
pfset TopoSlopesY.GeomNames ""

pfset TopoSlopesY.Geom.domain.Value 0.0


#---------------------------------------------------------
# Mannings coefficient 
#---------------------------------------------------------
# mannings roughnesses do not figure into the impes (fully sat) case but we still
# need a key for them

pfset Mannings.Type "Constant"
pfset Mannings.GeomNames ""
pfset Mannings.Geom.domain.Value 0.

#-----------------------------------------------------------------------------
# Phase sources:
#-----------------------------------------------------------------------------

pfset PhaseSources.Type                         Constant
pfset PhaseSources.GeomNames                    domain
pfset PhaseSources.Geom.domain.Value        0.0

pfset PhaseSources.water.Type                  Constant
pfset PhaseSources.water.GeomNames             domain
pfset PhaseSources.water.Geom.domain.Value 0.0


#-----------------------------------------------------------------------------
#  Solver Impes  
#-----------------------------------------------------------------------------
pfset Solver  Impes
pfset Solver.MaxIter 50
pfset Solver.AbsTol  1E-10
pfset Solver.Drop   1E-15

#-----------------------------------------------------------------------------
# Run and Unload the ParFlow output files
#-----------------------------------------------------------------------------

set n_runs 1


pfrun deadrun
pfundist deadrun

set mask [pfload deadrun.out.mask.pfb]
set top [Parflow::pfcomputetop $mask]

set data [pfload deadrun.out.press.pfb]
set top_data [Parflow::pfextracttop $top $data]

pfsave $top_data -sa "top.press.txt"

  set tmpname tmp
   set filename top.press.txt
    set source [open $filename]
    set destination [open $tmpname w]
    set content [read $source]
    close $source

    set lines [split $content \n]
    set lines_after_deletion \
       [lreplace $lines 0 0]

    puts -nonewline $destination [join $lines_after_deletion \n]
    close $destination
    file rename -force $tmpname $filename
