program mask_slopes

! aditi bhaskar (aditi.bhaskar@umbc.edu)
! last modified 5 august 2010
!
!  this code combines the pitfilling code from reed maxwell (fillsinks1_pitfilling.f90) and the make slopes code from reed maxwell (make_slopes.f90) 
!  to apply this code, change the first set of lines from ny, ny to the input and output file names.  the input dem file should be set up so the first row is the northern most row of cells in the domain.  so that the number of rows and columns are the nx and ny values in your domain.  this is the format of an ascii raster from arcgis, but remove the first 6 header lines.
!  
! this code does pit filling, and adds 0.15 m to every sink cell (this can be modified by searching for this line:    dem(i,j) = dem(i,j) + 0.15). 
!
! this code also contains lines SPECIFIC TO DEAD RUN: the slopes at some cells are hardwired for the dead run case.  if you are using this code for another purpose, search for the following lines and remove them:
!sx(57,66) = 0.01
!sx(56,66) = 0.01
!sx(90,58) = -0.01
!sy(3,33) = -0.01
!
! i am running this code with ifort (intel fortran).  if you want to change this gfortran, you can change the formatting parts to free format, like write (10, * instead of write(10,'(1000(f6.2,x))'
!
!  Note: Although you may be tempted to make x the left-right axis on your dem, this would make it the columns/i, but dem(i,j) means i should be rows. 
!  So make sure you enter nx as the number of rows and ny as columns. 

implicit none
real dx, dy, dz, datum,counter,mav, dem_min, dem_max,x
integer*4 nx, ny, nz,mi,mj,ii,jj,dummy,icount
integer lj,hj,li,hi,sflag,wsize,i,j,k,itcount,maxiter
real*8, allocatable :: grid(:,:,:),sx(:,:),sy(:,:),dem(:,:),dir(:,:),corr(:,:)
character*40 filename, dem_filename, filenamextxt, filenameytxt, filenamexpfb, filenameypfb, dem_output
character*2, allocatable :: string(:,:)
! ------  INPUT LINES -------

nx = 90
ny = 80
nz = 10
dx = 100.
dy = 100.
dz = 1.
filenamexpfb = 'xslope.pfb'  ! set names of the output pfb 
filenameypfb = 'yslope.pfb'
filenamextxt = 'xslope.txt'  ! set names of the output txt files
filenameytxt = 'yslope.txt'
dem_filename = 'dem_pcr.txt'  ! name of input dem txt file, format of an ascii raster from arcgis but remove the first 6 header lines
dem_output = 'ned_centralpit.txt'  ! name of output dem txt file
dummy=1
wsize = 1
maxiter=750
dem_min = 10000000.
dem_max = -1000.

allocate (grid(nx,ny,nz),sx(nx,ny),sy(nx,ny),dem(nx,ny),dir(nx,ny),string(nx,ny),corr(nx,ny))

open(667,file= trim(dem_output))

! read in dem
open(10, file = trim(dem_filename))
do i = 1, nx
	read(10,*) (dem(i,j), j=1,ny)
end do

datum = 250.
grid = 0.

! ------ END INPUT LINES -------

! loop over PF grid and set topo for CLM
! grid= 1 if w/in 10 nodes of Land Surface
! grid= 0 otherwise
!

sflag = 0
itcount = 0

do while (sflag == 0)
sflag=1
itcount = itcount + 1
!print *,"ITERATION",itcount
! loop over and calculate slopes
sx = 0.
sy = 0.

do i = 1, nx
    do j = 1, ny
    if (i==1) then
      sx(i,j) = (dem(i+1,j)-dem(i,j))/(dx)
    elseif(i==nx) then
      sx(i,j) = (dem(i,j)-dem(i-1,j))/(dx)
    else
      if(dem(i+1,j) > dem(i,j) .and. dem(i-1,j) > dem(i,j)) then
        sx(i,j) = 0.0d0
      else  
        sx(i,j) = (dem(i+1,j)-dem(i-1,j))/(2.*dx) ! central 
!	sx(i,j) = (dem(i+1,j)-dem(i,j))/(dx) ! forward 
!	sx(i,j) = (dem(i,j)-dem(i-1,j))/(dx) ! backward 
      endif
    endif
    end do !j
end do  ! i

do i = 1, nx
    do j = 1, ny
    if (j==1) then
      sy(i,j) = (dem(i,j+1)-dem(i,j))/(dy)
    elseif(j==ny) then
      sy(i,j) = (dem(i,j)-dem(i,j-1))/(dy)
    else
      if(dem(i,j+1) > dem(i,j) .and. dem(i,j-1) > dem(i,j)) then
        sy(i,j) = 0.0d0
      else  
        sy(i,j) = (dem(i,j+1)-dem(i,j-1))/(2.*dy) ! central
!        sy(i,j) = (dem(i,j+1)-dem(i,j))/(dy) ! forward
!        sy(i,j) = (dem(i,j)-dem(i,j-1))/(dy) ! backward
      endif
    endif
    end do !j
end do  ! i

!Compute flow direction
!do i=1,nx
!  do j=1,ny
!   dir(i,j)=sqrt((sx(i,j))*2.0d0+(sy(i,j)))*2.0d0
!   if(sx(i,j) < 0.0 .and. sy(i,j) < 0.0) string(i,j)="nw"
!   if(sx(i,j) > 0.0 .and. sy(i,j) > 0.0) string(i,j)="se"
!   if(sx(i,j) < 0.0 .and. sy(i,j) > 0.0) string(i,j)="sw"
!   if(sx(i,j) > 0.0 .and. sy(i,j) < 0.0) string(i,j)="ne"
!   if(sx(i,j) < 0.0 .and. sy(i,j) == 0.0) string(i,j)="ww"
!   if(sx(i,j) > 0.0 .and. sy(i,j) == 0.0) string(i,j)="ee"
!   if(sx(i,j) == 0.0 .and. sy(i,j) > 0.0) string(i,j)="ss"
!   if(sx(i,j) == 0.0 .and. sy(i,j) < 0.0) string(i,j)="nn"
!   if(sx(i,j) == 0.0 .and. sy(i,j) == 0.0) string(i,j)="oo"
!   enddo
!enddo

do i=1,nx
  do j=1,ny
   dir(i,j)=sqrt((sx(i,j))**2.0d0+(sy(i,j))**2.0d0)
   if(sx(i,j) < 0.0 .and. sy(i,j) < 0.0) string(i,j)="|-"
   if(sx(i,j) > 0.0 .and. sy(i,j) > 0.0) string(i,j)="-|"
   if(sx(i,j) < 0.0 .and. sy(i,j) > 0.0) string(i,j)="|-"
   if(sx(i,j) > 0.0 .and. sy(i,j) < 0.0) string(i,j)="-|"
   if(sx(i,j) < 0.0 .and. sy(i,j) == 0.0) string(i,j)="--"
   if(sx(i,j) > 0.0 .and. sy(i,j) == 0.0) string(i,j)="--"
   if(sx(i,j) == 0.0 .and. sy(i,j) > 0.0) string(i,j)="||"
   if(sx(i,j) == 0.0 .and. sy(i,j) < 0.0) string(i,j)="||"
   if(sx(i,j) == 0.0 .and. sy(i,j) == 0.0) string(i,j)="oo"
   enddo
enddo

!If there is a sink calc moving average
do i=1,nx
 do j=1,ny
   if (dir(i,j) == 0.0d0) then
   mav = 0.0
   counter = 0.0
   
   li = i - wsize
   hi = i + wsize
   lj = j - wsize
   hj = j + wsize
   if (j <= wsize) then
    lj = 1
   elseif (hj > (ny-wsize)) then
    hj = ny
   endif
   if (li <= wsize) then
    li = 1
   elseif (hi > (nx-wsize)) then
    hi = nx
   endif
    
!   do ii=li,hi
!     do jj=lj,hj
!       if (ii/=i.or.jj/=j) then
!       mav = mav + dem(ii,jj)
       counter = counter + 1.0
!       endif
!     enddo
!   enddo
!   mav = mav +0.15
!   mav = mav / (counter)
   dem(i,j) = dem(i,j) + 0.15
   endif
  enddo
 enddo
 
 ! Check if all sinks are filled
 do i = 1, nx
    do j = 1, ny
    if (i==1) then
      sx(i,j) = (dem(i+1,j)-dem(i,j))/(dx)
    elseif(i==nx) then
      sx(i,j) = (dem(i,j)-dem(i-1,j))/(dx)
    else
      if(dem(i+1,j) > dem(i,j) .and. dem(i-1,j) > dem(i,j)) then
        sx(i,j) = 0.0d0
      else  
        sx(i,j) = (dem(i+1,j)-dem(i-1,j))/(2.*dx) ! central 
!	sx(i,j) = (dem(i+1,j)-dem(i,j))/(dx) ! forward 
!	sx(i,j) = (dem(i,j)-dem(i-1,j))/(dx) ! backward 
      endif
    endif
    end do !j
end do  ! i

do i = 1, nx
    do j = 1, ny
    if (j==1) then
      sy(i,j) = (dem(i,j+1)-dem(i,j))/(dy)
    elseif(j==ny) then
      sy(i,j) = (dem(i,j)-dem(i,j-1))/(dy)
    else
      if(dem(i,j+1) > dem(i,j) .and. dem(i,j-1) > dem(i,j)) then
        sy(i,j) = 0.0d0
      else  
        sy(i,j) = (dem(i,j+1)-dem(i,j-1))/(2.*dy) ! central
!        sy(i,j) = (dem(i,j+1)-dem(i,j))/(dy) ! forward
!        sy(i,j) = (dem(i,j)-dem(i,j-1))/(dy) ! backward
      endif
    endif
    end do !j
end do  ! i

do i=1,nx
  do j=1,ny
   dir(i,j)=sqrt((sx(i,j))**2.0+(sy(i,j))**2.0)
   if (dir(i,j) == 0.0d0.and.itcount<=maxiter) then
     sflag = 0
!     print *,"sink at:",i,j,dem(i,j),counter
!	 print *, i-1,j, dem(i-1,j)
!	 print *, i,j-1, dem(i,j-1)
!	 print *, i,j+1, dem(i,j+1)
!	 print *, i+1,j, dem(i+1,j)
   endif  
   enddo
enddo

enddo !Iteration loop
 

do i=1,nx
 write(667,'(1000(f10.4,x))')(dem(i,j),j=1,ny)
enddo

!do j = 1, ny
!do i = 1, nx
! write(11,*) dem(i,j)
! if (dem(i,j) > dem_max) dem_max = dem(i,j)
! if (dem(i,j) < dem_min) dem_min = dem(i,j)
!end do
!end do
 
 print*, 'itcount',itcount
! print*, 'dem_min',dem_min
! print*, 'dem_max',dem_max

! set slopes to zero
sx = 0.
sy = 0.

! loop over and calculate slopes
do i = 1, nx
    do j = 1, ny
    if (i==1) then
      sx(i,j) = (dem(i+1,j)-dem(i,j))/(dx)
    elseif(i==nx) then
      sx(i,j) = (dem(i,j)-dem(i-1,j))/(dx)
    else
      if(dem(i+1,j) > dem(i,j) .and. dem(i-1,j) > dem(i,j)) then
        sx(i,j) = 0.0d0
      else  
        sx(i,j) = (dem(i+1,j)-dem(i-1,j))/(2.*dx)
      endif
    endif
    end do !j
end do  ! i

do i = 1, nx
    do j = 1, ny
    if (j==1) then
      sy(i,j) = (dem(i,j+1)-dem(i,j))/(dy)
    elseif(j==ny) then
      sy(i,j) = (dem(i,j)-dem(i,j-1))/(dy)
   else
     if(dem(i,j+1) > dem(i,j) .and. dem(i,j-1) > dem(i,j)) then
        sy(i,j) = 0.0d0
      else  
        sy(i,j) = (dem(i,j+1)-dem(i,j-1))/(2.*dy)
      endif
    endif
    end do !j
end do  ! i

! hardwire in the slopes at these places for dead run example
sx(57,66) = 0.01
sx(56,66) = 0.01
sx(90,58) = -0.01
sy(3,33) = -0.01
  
do i =1, nx
    do j = 1, ny
        grid(i,j,1:nz) = sx(i,j)   !x slopes
    end do
end do

! write out slopes
call pf_write(grid,filenamexpfb,nx,ny,nz,dx,dy,dz)

! write slopes to text file for error checking/plotting etc
open(666, file = trim(filenamextxt))

do i=1,nx
 write(666,'(1000(f10.4,x))')(sx(i,j),j=1,ny)
enddo
close(666)
print *,"X-Slopes successful"

do i =1, nx
    do j = 1, ny
        grid(i,j,1:nz) = sy(i,j)   ! y slopes
    end do
end do

! write slopes to text file for error checking/plotting etc
call pf_write(grid,filenameypfb,nx,ny,nz,dx,dy,dz)

open(666, file = trim(filenameytxt))
do i=1,nx
 write(666,'(1000(f10.4,x))')(sy(i,j),j=1,ny)
enddo
close(666)

print *,"Y-Slopes successful"
end program mask_slopes


subroutine pf_write(x,filename,ixlim,iylim,izlim,dx,dy,dz)
Real*8 x(ixlim,iylim,izlim),  ri, rj, rk1, rk2, headsum,    &
rsum, junk,                                           &   
ksum, kavg,f, dx, dy, dz, x1, y1, z1
Integer*4 i,j,k, i1, j1, k1, ixlim, iylim, izlim, &
ns, ix, iy, iz, rx, ry, rz
character*40 filename

!
!	Open File
!
filename = trim(filename)
open(15,file=filename,form='unformatted',&
access='stream',convert='BIG_ENDIAN')
!
! Calc domain bounds
!
	ix = 0
	iy = 0
	iz = 0


	ns = 1

	rx = 1
	ry = 1
	rz = 1

	x1 = dble(ixlim)*dx
	y1 = dble(iylim)*dy
	z1 = dble(izlim)*dz
!
!	Write header info
!

write(15) x1
write(15) y1
write(15) z1

write(15) ixlim
write(15) iylim
write(15) izlim

write(15) dx
write(15) dy
write(15) dz

write(15) ns

write(15) ix
write(15) iy
write(15) iz

write(15) ixlim
write(15) iylim
write(15) izlim

write(15) rx
write(15) ry
write(15) rz


do  k=1,izlim
do  j=1,iylim
do  i=1,ixlim
write(15) x(i,j,k)
end do
end do
end do

close(15)
return
end

