program pf_sol_gen

!
!  written by Reed Maxwell (maxwell5@llnl.gov)
!  10-30-04
!  
!  Comments by Aditi Bhaskar (aditi.bhaskar@umbc.edu).  
!
!  This code is used to make part of the X-Y domain inactive.
!  This code is similar to pf_solid_file_create.f90, except: 
!  It has 3 patches: "z-upper z-lower perimeter" 
!  It reads in a mask to determine what areas of the domain are inactive or active. 
!  To create this mask, make an ASCII text file in matrix form with 1's for inactive cells and 2's for active cells. 
!  Then use mattocol.m to transform this from matrix form to a column, which is what is read in in this form. 
!  The dem file is also read in column format, and this same code mattocol.m can be used to transform the 
!  DEM matrix to a column.  
!
real, allocatable:: DEM(:,:), points(:,:),bottom_elev(:,:)
integer, allocatable:: triangles(:,:), patches(:,:), num_tri_patches(:), &
					    mask(:,:)

real datum, x0, y0, dx, dy, xmax, ymax, mask_read
integer nx_dem, ny_dem, num_points, num_tri, num_patches, i, j, k, ii, jj, kk, &
num_solid, nt_y0, nt_ymax, nt_x0, nt_xmax, nt_surf, perimeter(5000,2), num_per_pts, &
pfind(9,2)

character*40 dem_filename, pfsol_filename, mask_filename

! code creates a triangulated pf solid file
! it reads in a DEM file for the shape of the top surface, but assumes that
! the rest of the domain is rectangularly shaped (and is the same size as the
! extents of the DEM)
!
! input block, should be read in and made more general

nx_dem = 236   ! num dem pnts in x
ny_dem = 224   ! num dem pnts in y

! now we need to read in the mask before we know how many triangles
! we will have.
! mask 1 = inactive
!      2 = active
allocate (mask(0:nx_dem+1,0:ny_dem+1))
mask = 1

mask_filename = 'maskcol.txt'
open(10, file = trim(mask_filename))
!read(10,*)
do j = 1, ny_dem
do i =1, nx_dem
read(10,*) mask_read 
mask(i,j) = nint(mask_read)
!print*, mask(i,j)
!mask(i,j)=2
end do
end do
close(10)

! determine perimeter nodes
! i=1, increase j until until mask = 2, then
! we sweep in a CC direction until mask goes from 1 to 2, then repeat
! ii = perimeter counter
ii = 1
i = 1
do j = 1, ny_dem
if (mask(i,j) == 2) exit
end do

perimeter(ii, 1) = i
perimeter(ii, 2) = j

pfind(1,1) = -1
pfind(1,2) =  0
pfind(2,1) = -1
pfind(2,2) = -1
pfind(3,1) =  0
pfind(3,2) = -1
pfind(4,1) =  1
pfind(4,2) = -1
pfind(5,1) =  1
pfind(5,2) =  0
pfind(6,1) =  1
pfind(6,2) =  1
pfind(7,1) =  0
pfind(7,2) =  1
pfind(8,1) = -1
pfind(8,2) =  1
pfind(9,1) = -1
pfind(9,2) =  0


do ii = 2, 5000
! sweep around peremiter
do jj = 2, 9
if ((mask(perimeter(ii-1,1) + pfind(jj,1),perimeter(ii-1,2) + pfind(jj,2)) -           &
     mask(perimeter(ii-1,1) + pfind(jj-1,1),perimeter(ii-1,2) + pfind(jj-1,2) )) == 1) &
     exit
end do  ! jj

perimeter(ii,1) = perimeter(ii-1,1) + pfind(jj,1)
perimeter(ii,2) = perimeter(ii-1,2) + pfind(jj,2)

if ((perimeter(ii,1) == perimeter(1,1)).and.(perimeter(ii,2) == perimeter(1,2))) exit

end do  ! ii

num_per_pts = ii

!do ii = 1, num_per_pts
!print*, ii, perimeter(ii,1), perimeter(ii,2)
!end do

x0 = 0.    ! origin coord, y
y0 = 0.    ! origin coord, x
dx = 500.00   ! dx
dy = 500.00   ! dy
xmax = float(nx_dem)*dx + x0
ymax = float(ny_dem)*dy + y0
datum = 0.0
!
num_solid = 1

! now we count up surface, L, R, F, B number of triangles and points
! to make the indexing easier, we dump out all points, so the 
! num points does not depend on the mask
! calculate total number of points
num_points =  2*nx_dem*ny_dem 

! currently we are gridding 2 triangles per box of four nodes, top and bottom
num_triangles = 4*(nx_dem-1)*(ny_dem-1)  + 4*(nx_dem-1) + 4*(ny_dem-1)

! y = y0 triangles


num_patches = 3

allocate(DEM(nx_dem,ny_dem),points(num_points,3),triangles(num_triangles,3),  &
patches(num_patches,num_triangles),num_tri_patches(num_triangles),bottom_elev(nx_dem,ny_dem) )
triangles = 1

bottom_elev = 0.  ! elevation of the bottom of the domain, init to 0.
dem = 0.  ! elevation of the top of the domain, init to 0.

dem_filename = 'demcol.txt'
pfsol_filename = 'full500.pfsol'

open(10, file = trim(dem_filename))
!read(10,*)
do j = 1, ny_dem
do i =1, nx_dem
read(10,*) dem(i,j)
dem(i,j) = dem(i,j) + 200
end do
end do
close(10)


!constant bottom, dem can be read from file
do j = 1, ny_dem
do i =1, nx_dem

 bottom_elev(i,j) = 0.0
end do
end do


! loop over points and assign, jj = point counter
! first the DEM/upper surface
! we assume points are in the center of the digital elevation tile 

jj = 1
do i = 1, nx_dem
do j = 1, ny_dem
points(jj,1) = float(i-1)*dx + dx/2. + x0
points(jj,2) = float(j-1)*dy + dy/2. + y0
points(jj,3) = dem(i,j)
jj = jj + 1
end do
end do

! then the lower surface; currently a flat surface - i.e. datum
do i = 1, nx_dem
do j = 1, ny_dem
points(jj,1) = float(i-1)*dx + dx/2. + x0
points(jj,2) = float(j-1)*dy + dy/2. + y0
points(jj,3) = bottom_elev(i,j)
jj = jj + 1
end do
end do

print*, 'points: ',jj - 1, num_points

!
jj = 1
ii = 1
do i = 1, (nx_dem-1)
do j = 1, (ny_dem-1)
!do j = jofi(i,1), (jofi(i,2)-1)

if ((mask(i,j) == 2).and.(mask(i+1,j+1)==2).and.(mask(i,j+1)==2)) then
triangles(jj,1) = (i-1)*ny_dem + j 
triangles(jj,2) = (i)*ny_dem + j  + 1
triangles(jj,3) = (i-1)*ny_dem + j  +1
patches(1,ii) = jj
jj = jj + 1
ii = ii + 1
end if

if ((mask(i,j) == 2).and.(mask(i+1,j)==2).and.(mask(i+1,j+1)==2)) then
triangles(jj,1) = (i-1)*ny_dem + j 
triangles(jj,2) = (i)*ny_dem + j 
triangles(jj,3) = (i)*ny_dem + j +1
patches(1,ii) = jj
jj = jj + 1
ii = ii + 1
end if

end do
end do

!loop around the perimeter of the upper surface to add edge triangles we've missed
!
do kk = 1, num_per_pts-1

!check for \| edge triangle
if ( (perimeter(kk+1,1)-perimeter(kk,1) == 1).and.(perimeter(kk+1,2)-perimeter(kk,2) == -1)) then
i = perimeter(kk,1)
j = perimeter(kk,2)
!print*, i, j
triangles(jj,1) =  (i-1)*ny_dem + j
triangles(jj,3) =  (i)*ny_dem + j
i = perimeter(kk+1,1)
j = perimeter(kk+1,2)
triangles(jj,2) =  (i-1)*ny_dem + j
patches(1,ii) = jj
jj = jj + 1
ii = ii + 1
end if

!check for |\ edge triangle
if ( (perimeter(kk+1,1)-perimeter(kk,1) == -1).and.(perimeter(kk+1,2)-perimeter(kk,2) == 1)) then
i = perimeter(kk,1)
j = perimeter(kk,2)
triangles(jj,1) =  (i-1)*ny_dem + j
triangles(jj,3) =  (i-2)*ny_dem + j
i = perimeter(kk+1,1)
j = perimeter(kk+1,2)
triangles(jj,2) =  (i-1)*ny_dem + j
patches(1,ii) = jj
jj = jj + 1
ii = ii + 1
end if



end do

num_tri_patches(1) = ii - 1
ii = 1


! then the lower surface; 
do i = 1, (nx_dem-1)
do j = 1, (ny_dem-1)
!do j = jofi(i,1), (jofi(i,2)-1)
if ((mask(i,j) == 2).and.(mask(i+1,j+1)==2).and.(mask(i,j+1)==2)) then
triangles(jj,1) = (i-1)*ny_dem + j + nx_dem*ny_dem
triangles(jj,2) = (i-1)*ny_dem + j  +1 + nx_dem*ny_dem
triangles(jj,3) = (i)*ny_dem + j + 1 + nx_dem*ny_dem
patches(2,ii) = jj
jj = jj + 1
ii = ii + 1
end if

if ((mask(i,j) == 2).and.(mask(i+1,j)==2).and.(mask(i+1,j+1)==2)) then
triangles(jj,1) = (i-1)*ny_dem + j + nx_dem*ny_dem
triangles(jj,2) = (i)*ny_dem + j +1 + nx_dem*ny_dem
triangles(jj,3) = (i)*ny_dem + j + nx_dem*ny_dem
patches(2,ii) = jj
jj = jj + 1
ii = ii + 1
end if
end do
end do

!loop around the perimeter of the lower surface to add edge triangles we've missed
! note these TRI's have to be specified in the opposite order as the top face
!
do kk = 1, num_per_pts-1

!check for \| edge triangle
if ( (perimeter(kk+1,1)-perimeter(kk,1) == 1).and.(perimeter(kk+1,2)-perimeter(kk,2) == -1)) then
i = perimeter(kk,1)
j = perimeter(kk,2)
triangles(jj,1) =  (i-1)*ny_dem + j + nx_dem*ny_dem
triangles(jj,2) =  (i)*ny_dem + j + nx_dem*ny_dem
i = perimeter(kk+1,1)
j = perimeter(kk+1,2)
triangles(jj,3) =  (i-1)*ny_dem + j + nx_dem*ny_dem
patches(2,ii) = jj
jj = jj + 1
ii = ii + 1
end if

!check for |\ edge triangle
if ( (perimeter(kk+1,1)-perimeter(kk,1) == -1).and.(perimeter(kk+1,2)-perimeter(kk,2) == 1)) then
i = perimeter(kk,1)
j = perimeter(kk,2)
triangles(jj,1) =  (i-1)*ny_dem + j + nx_dem*ny_dem
triangles(jj,2) =  (i-2)*ny_dem + j + nx_dem*ny_dem
i = perimeter(kk+1,1)
j = perimeter(kk+1,2)
triangles(jj,3) =  (i-1)*ny_dem + j + nx_dem*ny_dem
patches(2,ii) = jj
jj = jj + 1
ii = ii + 1
end if

end do

num_tri_patches(2) = ii - 1
ii = 1

!loop around the perimeter
do kk = 1, num_per_pts-1

i = perimeter(kk,1)
j = perimeter(kk,2)
triangles(jj,1) =  (i-1)*ny_dem + j + nx_dem*ny_dem
triangles(jj,3) =  (i-1)*ny_dem + j

i = perimeter(kk+1,1)
j = perimeter(kk+1,2)
triangles(jj,2) =  (i-1)*ny_dem + j 
patches(3,ii) = jj
jj = jj + 1
ii = ii + 1

i = perimeter(kk,1)
j = perimeter(kk,2)
triangles(jj,1) =  (i-1)*ny_dem + j + nx_dem*ny_dem
i = perimeter(kk+1,1)
j = perimeter(kk+1,2)
triangles(jj,3) =  (i-1)*ny_dem + j 
triangles(jj,2) =  (i-1)*ny_dem + j + nx_dem*ny_dem
patches(3,ii) = jj
jj = jj + 1
ii = ii + 1

end do !kk

num_tri_patches(3) = ii - 1
ii = 1


! then x=x0 face; 
!do j = 1, (ny_dem-1)
!do j = jofi(1,1), (jofi(1,2)-1)
!triangles(jj,1) =  j + nx_dem*ny_dem
!triangles(jj,2) =  j 
!triangles(jj,3) =  j +1
!patches(3,ii) = jj
!jj = jj + 1
!ii = ii + 1

!triangles(jj,1) =  j + nx_dem*ny_dem
!triangles(jj,2) =  j + 1 
!triangles(jj,3) =  j + 1 + nx_dem*ny_dem
!patches(3,ii) = jj
!jj = jj + 1
!ii = ii + 1
!end do

!num_tri_patches(3) = ii - 1
!ii = 1

! now the x=xmax face; 
!do j = 1, (ny_dem-1)
!do j = jofi(nx_dem,1), (jofi(nx_dem,2)-1)
!triangles(jj,1) =  (nx_dem-1)*ny_dem + j + nx_dem*ny_dem
!triangles(jj,2) =  (nx_dem-1)*ny_dem + j  +1
!triangles(jj,3) =  j + (nx_dem-1)*ny_dem 
!patches(4,ii) = jj
!jj = jj + 1
!ii = ii + 1

!triangles(jj,1) =  j + nx_dem*ny_dem + (nx_dem-1)*ny_dem 
!triangles(jj,2) =  j + 1 + (nx_dem-1)*ny_dem + nx_dem*ny_dem
!triangles(jj,3) =  j + 1 + (nx_dem-1)*ny_dem 
!patches(4,ii) = jj
!jj = jj + 1
!ii = ii + 1

!end do

!num_tri_patches(4) = ii - 1
!ii = 1


! and the y=y0 face; 
!do i = 1, (nx_dem-1)
!triangles(jj,1) =  (i-1)*ny_dem + 1 + nx_dem*ny_dem
!triangles(jj,2) =  (i)*ny_dem + 1 
!triangles(jj,3) =  (i-1)*ny_dem + 1 
!patches(5,ii) = jj
!jj = jj + 1
!ii = ii + 1

!triangles(jj,1) =  (i-1)*ny_dem + 1 + nx_dem*ny_dem
!triangles(jj,2) =  1 +(i)*ny_dem + nx_dem*ny_dem
!triangles(jj,3) =  1 +(i)*ny_dem 
!patches(5,ii) = jj
!jj = jj + 1
!ii = ii + 1

!end do

!num_tri_patches(5) = ii - 1
!ii = 1


! and the y=ymax face; 
!do i = 1, (nx_dem-1)
!triangles(jj,1) =  (i-1)*ny_dem + ny_dem + nx_dem*ny_dem
!triangles(jj,2) =  (i-1)*ny_dem + ny_dem 
!triangles(jj,3) =  (i)*ny_dem + ny_dem
!patches(6,ii) = jj
!jj = jj + 1
!ii = ii + 1

!triangles(jj,1) =  (i-1)*ny_dem + ny_dem + nx_dem*ny_dem
!triangles(jj,2) =  ny_dem +(i)*ny_dem  
!triangles(jj,3) =  ny_dem +(i)*ny_dem + nx_dem*ny_dem
!patches(6,ii) = jj
!jj = jj + 1
!ii = ii + 1

!end do

!num_tri_patches(6) = ii - 1
!ii = 1

print*, 'triangles: ',jj - 1, num_triangles
num_triangles = jj -1

open(20,file= trim(pfsol_filename))
! write version
write(20,'(i1)') 1
! write num vertices/points
write(20,'(i8)') num_points
! write points
do i = 1, num_points
write(20,'(3(f15.4,2x))') points(i,1), points(i,2), points(i,3)
end do
! currently we are assuming 1 solid 
write(20,'(i4)') num_solid
do k = 1, num_solid

! write num trianglesw
write(20,'(i8)') num_triangles
! write triangles
do i = 1, num_triangles
write(20,'(3(i8,2x))') triangles(i,1) -1, triangles(i,2) -1, triangles(i,3) -1
end do ! num_triangles

!write number of patches
write(20,'(i3)') num_patches
do i = 1, num_patches
! write num triangles in each patch
write(20,'(1x,i8)') num_tri_patches(i)
do j = 1, num_tri_patches(i)
write(20,'(i8)') patches(i,j) - 1
!write(20,'(<num_tri_patches(i)>(i8,2x))') patches(i,1:num_tri_patches(i)) - 1
!write(20,*) patches(i,1:num_tri_patches(i)) - 1
end do  ! j, num_tri_patch
end do  ! i, num_patch

end do ! k, num solids
close (20)

! write dots to check w/ chunk
open(30,file= 'chunk_check.pt.txt')
! write points
write(30,'(5(f15.4,2x))') points(1,1), points(1,2), points(1,3),1,0.
do i = 2, num_points
write(30,'(5(f15.4,2x))') points(i,1), points(i,2), points(i,3),1,1.
end do
close(30)

! write 3 line triangles segments to check w/ chunk
open(30,file= 'chunk_check.tri.txt')
! write triangles
do i = 1, num_patches 
do j =  1  , num_tri_patches(i) 
!if (i>nx_dem*ny_dem) ii = 2
!if (i>2*nx_dem*ny_dem) ii = 3
!if (i>2*nx_dem*ny_dem+ 2*(nx_dem-1)) ii = 4
!if (i>2*nx_dem*ny_dem+ 4*(nx_dem-1)) ii = 5
!if (i>2*nx_dem*ny_dem+ 4*(nx_dem-1)+2*(ny_dem-1)) ii = 6
!print*, i,j,patches(i,j), triangles(patches(i,j),1), triangles(patches(i,j),2), triangles(patches(i,j),3)
write(30,'(5(f15.4,2x))') points(triangles(patches(i,j),1),1), points(triangles(patches(i,j),1),2), points(triangles(patches(i,j),1),3),float(i),0.
write(30,'(5(f15.4,2x))') points(triangles(patches(i,j),2),1), points(triangles(patches(i,j),2),2), points(triangles(patches(i,j),2),3),float(i),float(i)
write(30,'(5(f15.4,2x))') points(triangles(patches(i,j),3),1), points(triangles(patches(i,j),3),2), points(triangles(patches(i,j),3),3),float(i),float(i)
write(30,'(5(f15.4,2x))') points(triangles(patches(i,j),1),1), points(triangles(patches(i,j),1),2), points(triangles(patches(i,j),1),3),float(i),float(i)
end do
end do
close(30)

open(40, file = 'chunk.mask.check.asc')
write(40,*) nx_dem, ny_dem, 1
do j = 1, ny_dem
do i =1, nx_dem
write(40,*) float(mask(i,j))
end do
end do
close(40)


end program pf_sol_gen


