# Deadrun

#
# Import the ParFlow TCL package
#
lappend auto_path $env(PARFLOW_DIR)/bin 
package require parflow
namespace import Parflow::*


set tcl_precision 16

#-----------------------------------------------------------------------------
# File input version number
#-----------------------------------------------------------------------------
pfset FileVersion 4

#-----------------------------------------------------------------------------
# Process Topology
#-----------------------------------------------------------------------------

pfset Process.Topology.P        1
pfset Process.Topology.Q        1
pfset Process.Topology.R        1

#-----------------------------------------------------------------------------
# Computational Grid
#-----------------------------------------------------------------------------
pfset ComputationalGrid.Lower.X                0.0
pfset ComputationalGrid.Lower.Y                0.0
pfset ComputationalGrid.Lower.Z                0.0

pfset ComputationalGrid.DX	                 100.
pfset ComputationalGrid.DY                     100.
pfset ComputationalGrid.DZ	                 1.

pfset ComputationalGrid.NX                      90
pfset ComputationalGrid.NY                      80
pfset ComputationalGrid.NZ                     339

#---------------------------------------------------------
# The Names of the GeomInputs
#---------------------------------------------------------

pfset GeomInput.Names                 "solidinput"
pfset GeomInput.solidinput.InputType  SolidFile
pfset GeomInput.solidinput.GeomNames  domain
pfset GeomInput.solidinput.FileName   deadrun.pfsol


pfset Geom.domain.Patches             "z-upper z-lower x-lower x-upper \
                                      y-lower y-upper"

#-----------------------------------------------------------------------------
# Perm
#-----------------------------------------------------------------------------
pfset Geom.Perm.Names "domain"

pfset Geom.domain.Perm.Type Constant
pfset Geom.domain.Perm.Value 0.166667

## 0.166667 m/hr makes these the units of everything that follows. 4 m/day = 4.6E-5 m/s = 0.16667 m/hr for K. value from harvey. 

pfset Perm.TensorType               TensorByGeom

pfset Geom.Perm.TensorByGeom.Names  "domain"

pfset Geom.domain.Perm.TensorValX  1.0
pfset Geom.domain.Perm.TensorValY  1.0
pfset Geom.domain.Perm.TensorValZ  1.0

#-----------------------------------------------------------------------------
# Relative Permeability
#-----------------------------------------------------------------------------

pfset Phase.RelPerm.Type               VanGenuchten
pfset Phase.RelPerm.GeomNames          "domain"

pfset Geom.domain.RelPerm.Alpha         3.5
pfset Geom.domain.RelPerm.N             2. 

#---------------------------------------------------------
# Saturation
#---------------------------------------------------------

pfset Phase.Saturation.Type              VanGenuchten
pfset Phase.Saturation.GeomNames         "domain"

pfset Geom.domain.Saturation.Alpha        3.5
pfset Geom.domain.Saturation.N            2.
pfset Geom.domain.Saturation.SRes         0.2
pfset Geom.domain.Saturation.SSat         1.0

#-----------------------------------------------------------------------------
# Specific Storage
#-----------------------------------------------------------------------------

pfset SpecificStorage.Type            Constant
pfset SpecificStorage.GeomNames       "domain"
pfset Geom.domain.SpecificStorage.Value 1.0e-4

#-----------------------------------------------------------------------------
# Phases
#-----------------------------------------------------------------------------

pfset Phase.Names "water"

pfset Phase.water.Density.Type	Constant
pfset Phase.water.Density.Value	1.0

pfset Phase.water.Viscosity.Type	Constant
pfset Phase.water.Viscosity.Value	1.0

#-----------------------------------------------------------------------------
# Contaminants
#-----------------------------------------------------------------------------
pfset Contaminants.Names			""


#-----------------------------------------------------------------------------
# Gravity
#-----------------------------------------------------------------------------

pfset Gravity				1.0

#-----------------------------------------------------------------------------
# Setup timing info
#-----------------------------------------------------------------------------

pfset TimingInfo.BaseUnit        0.01
pfset TimingInfo.StartCount      0
pfset TimingInfo.StartTime       0.0
pfset TimingInfo.StopTime        75.0
pfset TimingInfo.DumpInterval    -1
pfset TimeStep.Type              Constant
pfset TimeStep.Value             1.0


#-----------------------------------------------------------------------------
# Porosity
#-----------------------------------------------------------------------------

pfset Geom.Porosity.GeomNames          domain

pfset Geom.domain.Porosity.Type    Constant
pfset Geom.domain.Porosity.Value   0.39

#-----------------------------------------------------------------------------
# Domain
#-----------------------------------------------------------------------------
pfset Domain.GeomName domain

#-----------------------------------------------------------------------------
# Mobility
#-----------------------------------------------------------------------------
pfset Phase.water.Mobility.Type        Constant
pfset Phase.water.Mobility.Value       1.0


#-----------------------------------------------------------------------------
# Wells
#-----------------------------------------------------------------------------
pfset Wells.Names ""


#-----------------------------------------------------------------------------
# Time Cycles
#-----------------------------------------------------------------------------
pfset Cycle.Names "constant"
pfset Cycle.constant.Names              "alltime"
pfset Cycle.constant.alltime.Length      1
pfset Cycle.constant.Repeat             -1

 
#-----------------------------------------------------------------------------
# Boundary Conditions: Pressure
#-----------------------------------------------------------------------------
pfset BCPressure.PatchNames          "z-upper z-lower x-lower x-upper \
                                      y-lower y-upper"

pfset Patch.y-lower.BCPressure.Type			FluxConst
pfset Patch.y-lower.BCPressure.Cycle			"constant"
pfset Patch.y-lower.BCPressure.alltime.Value		0.0

pfset Patch.y-upper.BCPressure.Type			FluxConst
pfset Patch.y-upper.BCPressure.Cycle			"constant"
pfset Patch.y-upper.BCPressure.alltime.Value		0.0

pfset Patch.x-lower.BCPressure.Type			FluxConst
pfset Patch.x-lower.BCPressure.Cycle			"constant"
pfset Patch.x-lower.BCPressure.alltime.Value		0.0

pfset Patch.x-upper.BCPressure.Type			FluxConst
pfset Patch.x-upper.BCPressure.Cycle			"constant"
pfset Patch.x-upper.BCPressure.alltime.Value		0.0

pfset Patch.z-lower.BCPressure.Type			FluxConst
pfset Patch.z-lower.BCPressure.Cycle			"constant"
pfset Patch.z-lower.BCPressure.alltime.Value		0.0

pfset Patch.z-upper.BCPressure.Type		      OverlandFlow
pfset Patch.z-upper.BCPressure.Cycle		      "constant"
pfset Patch.z-upper.BCPressure.alltime.Value		0.0

#---------------------------------------------------------
# Topo slopes in x-direction
#---------------------------------------------------------

pfset TopoSlopesX.Type "PFBFile"
pfset TopoSlopesX.GeomNames "domain"
pfset TopoSlopesX.FileName xslopeenf_full.pfb

#---------------------------------------------------------
# Topo slopes in y-direction
#---------------------------------------------------------

pfset TopoSlopesY.Type "PFBFile"
pfset TopoSlopesY.GeomNames "domain"
pfset TopoSlopesY.FileName yslopeenf_full.pfb


#---------------------------------------------------------
# Mannings coefficient 
#---------------------------------------------------------

pfset Mannings.Type "Constant"
pfset Mannings.GeomNames "domain"
pfset Mannings.Geom.domain.Value 2.75e-6

#---------------------------------------------------------
# Initial conditions: water pressure
#---------------------------------------------------------

pfset ICPressure.Type                                   HydroStaticPatch
pfset ICPressure.GeomNames                              domain
pfset Geom.domain.ICPressure.Value                      -5.0
pfset Geom.domain.ICPressure.RefGeom                    domain
pfset Geom.domain.ICPressure.RefPatch                   z-upper

#-----------------------------------------------------------------------------
# Phase sources:
#-----------------------------------------------------------------------------

pfset PhaseSources.Type                         Constant
pfset PhaseSources.GeomNames                    domain
pfset PhaseSources.Geom.domain.Value        0.0

pfset PhaseSources.water.Type                  Constant
pfset PhaseSources.water.GeomNames             domain
pfset PhaseSources.water.Geom.domain.Value 0.0

#-----------------------------------------------------------------------------
# Exact solution specification for error calculations
#-----------------------------------------------------------------------------

pfset KnownSolution                                    NoKnownSolution

#-----------------------------------------------------------------------------
# Set solver parameters
#-----------------------------------------------------------------------------

pfset Solver                                             Richards
pfset Solver.MaxIter                                     2500

pfset Solver.Nonlinear.MaxIter                           300
pfset Solver.Nonlinear.ResidualTol                       1e-4
pfset Solver.Nonlinear.EtaChoice                         Walker1 
# pfset Solver.Nonlinear.EtaChoice                         EtaConstant
# pfset Solver.Nonlinear.EtaValue                          0.001
pfset Solver.Nonlinear.UseJacobian                       False
pfset Solver.Nonlinear.DerivativeEpsilon                 1e-14
pfset Solver.Nonlinear.StepTol				 1e-10
pfset Solver.Nonlinear.Globalization                     LineSearch
pfset Solver.Linear.KrylovDimension                      20
pfset Solver.Linear.MaxRestart                           2

pfset Solver.Linear.Preconditioner                       MGSemi
pfset Solver.Linear.Preconditioner.MGSemi.MaxIter        5
pfset Solver.Linear.Preconditioner.MGSemi.MaxLevels      10
pfset Solver.PrintSubsurf				False
pfset  Solver.Drop                                      1E-20
pfset Solver.AbsTol                                     1E-12

pfset Solver.WriteSiloSubsurfData                       True
pfset Solver.WriteSiloSlopes                            True
pfset Solver.WriteSiloMask                              True
pfset Solver.WriteSiloMannings                          True
pfset Solver.WriteSiloSpecificStorage                   True


#-----------------------------------------------------------------------------
# Run and Unload the ParFlow output files
#-----------------------------------------------------------------------------

set n_runs 1


pfdist xslopeenf_full.pfb
pfdist yslopeenf_full.pfb

pfrun deadrun
pfundist deadrun

pfundist xslopeenf_full.pfb
pfundist yslopeenf_full.pfb

set mask [pfload deadrun.out.mask.pfb]
set top [Parflow::pfcomputetop $mask]

set data [pfload deadrun.out.press.00075.pfb]
set top_data [Parflow::pfextracttop $top $data]

pfsave $top_data -sa "top.press.txt"

set data [pfload deadrun.out.satur.00075.pfb]
set top_data [Parflow::pfextracttop $top $data]

pfsave $top_data -sa "top.satur.txt"

    set tmpname tmp
   set filename top.pressure.txt
    set source [open $filename]
    set destination [open $tmpname w]
    set content [read $source]
    close $source

    set lines [split $content \n]
    set lines_after_deletion \
       [lreplace $lines 0 0]

    puts -nonewline $destination [join $lines_after_deletion \n]
    close $destination
    file rename -force $tmpname $filename

    set tmpname tmp
   set filename top.satur.txt
    set source [open $filename]
    set destination [open $tmpname w]
    set content [read $source]
    close $source

    set lines [split $content \n]
    set lines_after_deletion \
       [lreplace $lines 0 0]

    puts -nonewline $destination [join $lines_after_deletion \n]
    close $destination
    file rename -force $tmpname $filename

set runname deadrun
set slope_x          [pfload $runname.out.slope_x.silo]
set slope_y          [pfload $runname.out.slope_y.silo]
set mannings         [pfload $runname.out.mannings.silo]
set specific_storage [pfload $runname.out.specific_storage.silo]
set porosity         [pfload $runname.out.porosity.silo]

set mask             [pfload $runname.out.mask.silo]
set top              [pfcomputetop $mask]

set surface_area_of_domain [expr [pfget ComputationalGrid.DX] * [pfget ComputationalGrid.DY] * [pfget ComputationalGrid.NX] * [pfget ComputationalGrid.NY]]

set prev_total_water_balance 0.0
set verbose 1

for {set i 0} {$i <= 75} {incr i} {
    if $verbose {
	puts "======================================================"
	puts "Timestep $i"
	puts "======================================================"
    }
    set total_water_in_domain 0.0

    set filename [format "%s.out.press.%05d.pfb" $runname $i]
    set pressure [pfload $filename]
    set surface_storage [pfsurfacestorage $top $pressure]
 #   pfsave $surface_storage -silo "surface_storage.$i.silo"
    set total_surface_storage [pfsum $surface_storage]
    if $verbose {
	puts [format "Surface storage\t\t\t\t\t : %.16e" $total_surface_storage]
    }
    set total_water_in_domain [expr $total_water_in_domain + $total_surface_storage]

    set filename [format "%s.out.satur.%05d.pfb" $runname $i]
    set saturation [pfload $filename]

    set subsurface_storage [pfsubsurfacestorage $mask $porosity $pressure $saturation $specific_storage]
 #   pfsave $subsurface_storage -silo "subsurface_storage.$i.silo"

    set total_subsurface_storage [pfsum $subsurface_storage]


    if $verbose {
	puts [format "Subsurface storage\t\t\t\t : %.16e" $total_subsurface_storage]
    }
    set total_water_in_domain [expr $total_water_in_domain + $total_subsurface_storage]

    if $verbose {
	puts [format "Total water in domain\t\t\t\t : %.16e" $total_water_in_domain]
	puts ""
    }

    set total_surface_runoff 0.0
    if { $i > 0} {
	set surface_runoff [pfsurfacerunoff $top $slope_x $slope_y $mannings $pressure]
#	pfsave $surface_runoff -silo "surface_runoff.$i.silo"
	set total_surface_runoff [expr [pfsum $surface_runoff] * [pfget TimeStep.Value]]
	if $verbose {
	    puts [format "Surface runoff from pftools\t\t\t : %.16e" $total_surface_runoff]
	}

    }

    if [expr $i < 11] {
	set bc_flux [pfget Patch.z-upper.BCPressure.rain.Value]
    } {
	set bc_flux [pfget Patch.z-upper.BCPressure.rec.Value]
    }

    set boundary_flux [expr $bc_flux * $surface_area_of_domain * [pfget TimeStep.Value]]
    if $verbose {
	puts [format "BC flux\t\t\t\t\t\t : %.16e" $boundary_flux]
    }

    # Note flow into domain is negative
    set expected_difference [expr $boundary_flux + $total_surface_runoff]
    if $verbose {
	puts [format "Total Flux\t\t\t\t\t : %.16e" $expected_difference]
    }

    if { $i > 0 } {

	if $verbose {
	    puts ""
	    puts [format "Diff from prev total\t\t\t\t : %.16e" [expr $total_water_in_domain - $prev_total_water_balance]]
	}

	if [expr $expected_difference != 0.0] {
	    set percent_diff [expr (abs(($prev_total_water_balance - $total_water_in_domain) - $expected_difference)) / abs($expected_difference) * 100]
	    if $verbose {
		puts [format "Percent diff from expected difference\t\t : %.12e" $percent_diff]
	    }
	}

	set expected_water_balance [expr $prev_total_water_balance - $expected_difference]
	set percent_diff [expr abs(($total_water_in_domain - $expected_water_balance)) / $expected_water_balance * 100]
	if $verbose {
	    puts [format "Percent diff from expected total water sum\t : %.12e" $percent_diff]
	}

	if [expr $percent_diff > 0.005] {
	    puts "Error: Water balance is not correct"
	    set passed 0
	}

    }

    set prev_total_water_balance [expr $total_water_in_domain]
}




